SystemOrganization addCategory: #'ICU-Core'!

!String methodsFor: '*icu-core' stamp: 'JanVanDeSandt 2/21/2012 22:43'!
asUTF16Bytes

	^ self asUTF16String bytes! !

!String methodsFor: '*icu-core' stamp: 'JanVanDeSandt 2/21/2012 22:18'!
asUTF16String 

	| ustring |
	
	self isTwoByteString
		ifTrue: [ ^ self ].

	ustring := TwoByteString new: self size.
	self withIndexDo: [ :char :index | ustring at: index put: char ].
	^ustring! !

!String methodsFor: '*icu-core' stamp: 'JanVanDeSandt 2/21/2012 22:18'!
isTwoByteString

	^ false! !

String subclass: #TwoByteString
	instanceVariableNames: 'bytes'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ICU-Core'!

!TwoByteString class methodsFor: 'as yet unclassified' stamp: 'JanVanDeSandt 2/21/2012 22:02'!
fromByteArray: aByteArray 

	aByteArray size \\ 2 = 0 ifFalse: [^ ByteString fromByteArray: aByteArray ].
	^ self basicNew setBytes: aByteArray! !

!TwoByteString class methodsFor: 'instance creation' stamp: 'JanVanDeSandt 2/21/2012 22:10'!
fromByteArray: aByteArray charSize: anInteger

	aByteArray size = (anInteger*2)
		ifTrue: [ ^ self fromByteArray: aByteArray ].
		
	^self basicNew setBytes: (aByteArray copyFrom: 1 to: anInteger*2)! !

!TwoByteString class methodsFor: 'as yet unclassified' stamp: 'JanVanDeSandt 2/21/2012 21:56'!
new: aSize

	^ self basicNew setBytes: (ByteArray new: aSize * 2)! !

!TwoByteString methodsFor: 'accessing' stamp: 'JanVanDeSandt 2/21/2012 21:52'!
at: index

	^ Character value: (bytes at: index*2 - 1) + ((bytes at: index*2) << 8)! !

!TwoByteString methodsFor: 'accessing' stamp: 'JanVanDeSandt 2/21/2012 21:59'!
at: index put: aCharacter

	bytes at: (index*2 - 1) put: aCharacter asInteger \\ 256.
	bytes at: (index*2) put: aCharacter asInteger >> 8.
	
	^ aCharacter! !

!TwoByteString methodsFor: 'accessing' stamp: 'JanVanDeSandt 3/2/2012 15:03'!
basicAt: index

	^ (bytes at: index*2 - 1) + ((bytes at: index*2) << 8)! !

!TwoByteString methodsFor: 'accessing' stamp: 'JanVanDeSandt 2/21/2012 22:44'!
bytes

	^ bytes! !

!TwoByteString methodsFor: 'testing' stamp: 'JanVanDeSandt 2/21/2012 22:19'!
isTwoByteString

	^ true! !

!TwoByteString methodsFor: 'initialize-release' stamp: 'JanVanDeSandt 2/21/2012 21:46'!
setBytes: aByteArray

	bytes := aByteArray! !

!TwoByteString methodsFor: 'accessing' stamp: 'JanVanDeSandt 2/21/2012 21:47'!
size

	^ bytes size / 2! !

Error subclass: #ICUError
	instanceVariableNames: ''
	classVariableNames: 'ErrorTextDictionary'
	poolDictionaries: ''
	category: 'ICU-Core'!

ICUError subclass: #ICUBufferOverflowError
	instanceVariableNames: 'length'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ICU-Core'!

!ICUError methodsFor: 'as yet unclassified' stamp: 'jvds 2/26/2012 15:47'!
clear

	ErrorTextDictionary := nil! !

!ICUError methodsFor: 'as yet unclassified' stamp: 'jvds 2/26/2012 15:46'!
errorTextDictionary

	^ ErrorTextDictionary  ifNil: [
		ErrorTextDictionary := Dictionary new
			at: 1 put: 'U_ILLEGAL_ARGUMENT_ERROR' ;
			at: 3 put: 'U_INVALID_FORMAT_ERROR' ;
			at: 15 put: 'U_BUFFER_OVERFLOW_ERROR' ;
			yourself ]! !

SharedPool subclass: #ICUConstants
	instanceVariableNames: ''
	classVariableNames: 'UCAL_DATE UCAL_DAY_OF_YEAR UCAL_DST_OFFSET UCAL_ERA UCAL_MONTH UCAL_WEEK_OF_MONTH UCAL_WEEK_OF_YEAR UCAL_YEAR UCAL_ZONE_OFFSET'
	poolDictionaries: ''
	category: 'ICU-Core'!

!ICUConstants commentStamp: '<historical>' prior: 0!
An ICUConstants is a pool dictionary with ICU related constants.!

!ICUConstants class methodsFor: 'initialization' stamp: 'jvds 6/13/2011 21:12'!
initialize

	self initializeCalendarConstants ! !

!ICUConstants class methodsFor: 'initialization' stamp: 'jvds 6/13/2011 21:24'!
initializeCalendarConstants

	UCAL_ERA := 0.
	UCAL_YEAR := 1.
	UCAL_MONTH := 2.
	UCAL_WEEK_OF_YEAR := 3.
	UCAL_WEEK_OF_MONTH := 4.
	UCAL_DATE := 5.
	UCAL_DAY_OF_YEAR := 6.
	
	UCAL_ZONE_OFFSET := 15.
	UCAL_DST_OFFSET := 16
! !

Object subclass: #ICUInterface
	instanceVariableNames: ''
	classVariableNames: 'Current'
	poolDictionaries: ''
	category: 'ICU-Core'!

!ICUInterface commentStamp: '<historical>' prior: 0!
An abstract class that provides access to the ICU native classes. Concrete subclasses provide the actual implementation. One of these subclasses can be the Current.!

!ICUInterface class methodsFor: 'accessing' stamp: 'JanVanDeSandt 3/2/2012 14:16'!
clearCurrent
	"Reset the current variable."
	
	Current := nil
	! !

!ICUInterface class methodsFor: 'accessing' stamp: 'JanVanDeSandt 3/2/2012 14:07'!
current
	"Answer a concrete subclass of the receiver"

	^ Current! !

!ICUInterface class methodsFor: 'testing' stamp: 'JanVanDeSandt 3/2/2012 14:21'!
isCurrent

	^ Current = self! !

!ICUInterface class methodsFor: 'accessing' stamp: 'JanVanDeSandt 3/2/2012 14:16'!
makeCurrent
	"Make the receiver the current interface class."
	
	Current := self
	! !

!ICUInterface class methodsFor: 'resources' stamp: 'JanVanDeSandt 3/2/2012 14:01'!
nativeCalendarClass

	self subclassResponsibility ! !

!ICUInterface class methodsFor: 'resources' stamp: 'JanVanDeSandt 3/2/2012 14:01'!
nativeConverterClass

	self subclassResponsibility ! !

Object subclass: #ICUModules
	instanceVariableNames: 'platformName commonModule i18nModule versionSuffix'
	classVariableNames: 'Default'
	poolDictionaries: ''
	category: 'ICU-Core'!

!ICUModules commentStamp: 'JanVanDeSandt 2/25/2012 15:32' prior: 0!
ICUModules contains the ICU4C library/module names and a possible versionSuffix.

The ICU - International Components for Unicode Library (icu4c) can be downloaded from: http://site.icu-project.org/

Win32
	Add the ICU bin directory to the PATH environment variable.
	Note that the ICU libraries on Windows require the Microsoft Visual C++ 2010 Redistributable Package

MaxOSX
	Get the 32bit versions of the ICU libraries
	For example by using MacPorts (on Snow Leapard set the build_arch configuration property to i386 to get 32bit libraries).
	Create a symbolic link to the libraries from inside the Resources directory of the VM. For example
		ln -s /opt/local/lib/libicuuc.dylib .
		ln -s /opt/local/lib/libicui18n .
	
Unix
	On Ubuntu 10.10 you can install the library:
		sudo apt-get install libicu42!

!ICUModules class methodsFor: 'accessing' stamp: 'JanVanDeSandt 2/25/2012 15:48'!
clear

	Default := nil! !

!ICUModules class methodsFor: 'accessing' stamp: 'JanVanDeSandt 2/25/2012 15:55'!
default

	^ Default ifNil: [ Default := self determineDefault ]! !

!ICUModules class methodsFor: 'accessing' stamp: 'JanVanDeSandt 2/25/2012 15:48'!
default: anObject

	Default := anObject! !

!ICUModules class methodsFor: 'accessing' stamp: 'JanVanDeSandt 2/25/2012 15:55'!
determineDefault

	Smalltalk os isWin32 
		ifTrue: [ ^ self winVersion48 ].
		
	Smalltalk os isMacOSX
		ifTrue: [ ^ self osxVersion48 ].
		
	^ self unixVersion42! !

!ICUModules class methodsFor: 'class initialization' stamp: 'JanVanDeSandt 2/25/2012 16:06'!
initialize 
	
	Smalltalk addToStartUpList: self! !

!ICUModules class methodsFor: 'resources' stamp: 'JanVanDeSandt 2/25/2012 16:08'!
osxVersion46
	"self default: self osxVersion46"

	^self new setCommonModule: 'libicuuc.dylib' i18nModule: 'libicui18n.dylib' versionSuffix: '_46'! !

!ICUModules class methodsFor: 'resources' stamp: 'JanVanDeSandt 2/25/2012 16:09'!
osxVersion48
	"self default: self osxVersion48"

	^self new setCommonModule: 'libicuuc.dylib' i18nModule: 'libicui18n.dylib' versionSuffix: '_48'! !

!ICUModules class methodsFor: 'class initialization' stamp: 'JanVanDeSandt 2/25/2012 16:00'!
startUp: resuming
	"Clear the default instance if it was created for another platform."

	resuming
		ifTrue: [
			(Default notNil and: [ Default platformName ~= Smalltalk os platformName ])
				ifTrue: [ Default := nil ] ]! !

!ICUModules class methodsFor: 'resources' stamp: 'JanVanDeSandt 2/25/2012 16:09'!
unixVersion42
	"self default: self unixVersion42"

	^self new setCommonModule: 'libicuuc.so.42' i18nModule: 'libicui18n.so.42' versionSuffix: '_4_2'! !

!ICUModules class methodsFor: 'resources' stamp: 'JanVanDeSandt 3/3/2012 23:55'!
unixVersion44
	"self default: self unixVersion44"

	^self new setCommonModule: 'libicuuc.so.44' i18nModule: 'libicui18n.so.44' versionSuffix: '_44'! !

!ICUModules class methodsFor: 'class initialization' stamp: 'JanVanDeSandt 3/2/2012 11:08'!
unload 

	self clear.
	Smalltalk removeFromStartUpList: self! !

!ICUModules class methodsFor: 'resources' stamp: 'JanVanDeSandt 2/25/2012 16:09'!
winVersion48
	"self default: self winVersion48"

	^self new setCommonModule: 'icuuc48.dll' i18nModule: 'icuin48.dll' versionSuffix: '_48'! !

!ICUModules methodsFor: 'accessing' stamp: 'JanVanDeSandt 2/25/2012 15:51'!
commonModule

	^commonModule ! !

!ICUModules methodsFor: 'testing' stamp: 'JanVanDeSandt 3/7/2012 08:22'!
hasVersionSuffix

	^versionSuffix notNil and: [ versionSuffix notEmpty ]! !

!ICUModules methodsFor: 'accessing' stamp: 'JanVanDeSandt 2/25/2012 15:51'!
i18nModule

	^ i18nModule ! !

!ICUModules methodsFor: 'accessing' stamp: 'JanVanDeSandt 2/25/2012 15:51'!
platformName

	^ platformName ! !

!ICUModules methodsFor: 'initialize-release' stamp: 'JanVanDeSandt 2/25/2012 15:50'!
setCommonModule: aCommonModuleName i18nModule: anI18NModuleName versionSuffix: aVersionSuffix

	platformName := Smalltalk os platformName.
	commonModule := aCommonModuleName.
	i18nModule := anI18NModuleName.
	versionSuffix := aVersionSuffix ! !

!ICUModules methodsFor: 'accessing' stamp: 'JanVanDeSandt 2/25/2012 15:52'!
versionSuffix

	^ versionSuffix! !

Object subclass: #ICUObject
	instanceVariableNames: 'native'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ICU-Core'!

ICUObject subclass: #ICUCalendar
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: 'ICUConstants'
	category: 'ICU-Core'!

!ICUCalendar class methodsFor: 'time zones' stamp: 'JanVanDeSandt 3/2/2012 14:34'!
allTimeZones

	^ self nativeClass allTimeZones ! !

!ICUCalendar class methodsFor: 'time zones' stamp: 'JanVanDeSandt 3/2/2012 14:33'!
defaultTimeZone

	^ self nativeClass getDefaultTimeZone! !

!ICUCalendar class methodsFor: 'resources' stamp: 'JanVanDeSandt 3/2/2012 14:09'!
nativeClass

	^ self icuInterface nativeCalendarClass ! !

!ICUCalendar class methodsFor: 'instance creation' stamp: 'JanVanDeSandt 3/2/2012 14:39'!
new

	^ self newWithLocale: nil timeZone: self defaultTimeZone ! !

!ICUCalendar class methodsFor: 'instance creation' stamp: 'JanVanDeSandt 3/2/2012 14:53'!
newWithLocale: aLocaleString timeZone: aTimeZoneString

	^ self basicNew setNative: (self nativeClass openDefaultWithLocale: aLocaleString timezone: aTimeZoneString)! !

!ICUCalendar methodsFor: 'as yet unclassified' stamp: 'JanVanDeSandt 3/2/2012 14:42'!
dayOfYear

	^self get: UCAL_DAY_OF_YEAR! !

!ICUCalendar methodsFor: 'as yet unclassified' stamp: 'JanVanDeSandt 3/2/2012 14:42'!
dstOffset
	"Field number indicating the daylight savings offset in milliseconds. "
	
	^self get: UCAL_DST_OFFSET ! !

!ICUCalendar methodsFor: 'as yet unclassified' stamp: 'JanVanDeSandt 3/2/2012 14:43'!
get: ucoConstant

	^ native get: ucoConstant! !

!ICUCalendar methodsFor: 'as yet unclassified' stamp: 'JanVanDeSandt 3/2/2012 14:43'!
zoneOffset
	"Field number indicating the daylight savings offset in milliseconds. "
	
	^self get: UCAL_ZONE_OFFSET ! !

ICUObject subclass: #ICUConverter
	instanceVariableNames: ''
	classVariableNames: 'UTF8Converter'
	poolDictionaries: ''
	category: 'ICU-Core'!

!ICUConverter commentStamp: '<historical>' prior: 0!
An ICUConverter is a converter ...!

!ICUConverter class methodsFor: 'as yet unclassified' stamp: 'jvds 2/26/2012 14:04'!
allNames

	^ self nativeClass allNames! !

!ICUConverter class methodsFor: 'converting' stamp: 'jvds 2/26/2012 15:09'!
byteStringAsUTF8: aByteString

	^ self utf8Converter fromType: self defaultByteStringConverterType with: aByteString! !

!ICUConverter class methodsFor: 'converting' stamp: 'JanVanDeSandt 3/4/2012 15:15'!
byteStringFromUTF8: aSource

	| buffer length |
	
	buffer := ByteArray new: aSource byteSize.

	length := self utf8Converter toType: self defaultByteStringConverterType source: aSource destination: buffer.
	^(ByteString new: length) replaceFrom: 1 to: length with: buffer startingAt: 1! !

!ICUConverter class methodsFor: 'as yet unclassified' stamp: 'jvds 2/26/2012 20:52'!
clear
	"self clear"

	UTF8Converter := nil! !

!ICUConverter class methodsFor: 'resources' stamp: 'jvds 2/26/2012 15:10'!
converterTypeLatin1

	^ 3! !

!ICUConverter class methodsFor: 'resources' stamp: 'jvds 2/26/2012 15:10'!
converterTypeUTF16BE

	^ 5! !

!ICUConverter class methodsFor: 'resources' stamp: 'jvds 2/26/2012 15:10'!
converterTypeUTF16LE

	^ 6! !

!ICUConverter class methodsFor: 'resources' stamp: 'jvds 2/26/2012 15:32'!
converterTypeUTF32

	^ 30! !

!ICUConverter class methodsFor: 'resources' stamp: 'jvds 2/26/2012 15:10'!
converterTypeUTF32BE

	^ 7! !

!ICUConverter class methodsFor: 'resources' stamp: 'jvds 2/26/2012 15:10'!
converterTypeUTF32LE

	^ 8! !

!ICUConverter class methodsFor: 'resources' stamp: 'jvds 2/26/2012 15:09'!
converterTypeUTF8

	^ 4! !

!ICUConverter class methodsFor: 'resources' stamp: 'jvds 2/26/2012 15:09'!
defaultByteStringConverterType

	^ self converterTypeLatin1! !

!ICUConverter class methodsFor: 'as yet unclassified' stamp: 'JanVanDeSandt 3/4/2012 14:15'!
defaultName

	^ self nativeClass getDefaultName! !

!ICUConverter class methodsFor: 'class initialization' stamp: 'jvds 2/26/2012 20:53'!
initialize

	Smalltalk addToShutDownList: self! !

!ICUConverter class methodsFor: 'resources' stamp: 'JanVanDeSandt 3/2/2012 14:09'!
nativeClass

	^ self icuInterface nativeConverterClass ! !

!ICUConverter class methodsFor: 'as yet unclassified' stamp: 'jvds 2/26/2012 14:02'!
open: aConverterName

	^ self new setNative: (self nativeClass open: aConverterName)! !

!ICUConverter class methodsFor: 'as yet unclassified' stamp: 'jvds 2/26/2012 14:03'!
openISO_8859_1

	^ self open: 'ISO-8859-1'! !

!ICUConverter class methodsFor: 'as yet unclassified' stamp: 'JanVanDeSandt 3/4/2012 14:47'!
openLatin1

	^ self openISO_8859_1! !

!ICUConverter class methodsFor: 'as yet unclassified' stamp: 'jvds 2/26/2012 14:03'!
openUTF32

	^ self open: 'UTF-32'! !

!ICUConverter class methodsFor: 'as yet unclassified' stamp: 'jvds 2/26/2012 14:04'!
openUTF8

	^ self open: 'UTF-8'! !

!ICUConverter class methodsFor: 'class initialization' stamp: 'jvds 2/26/2012 20:54'!
shutDown: quitting

	self clear! !

!ICUConverter class methodsFor: 'class initialization' stamp: 'JanVanDeSandt 3/2/2012 15:26'!
unload

	Smalltalk removeFromShutDownList: self! !

!ICUConverter class methodsFor: 'as yet unclassified' stamp: 'jvds 2/26/2012 15:08'!
utf8Converter

	^ UTF8Converter ifNil: [ UTF8Converter := self openUTF8 ]! !

!ICUConverter methodsFor: 'actions' stamp: 'JanVanDeSandt 3/4/2012 14:30'!
fromType: aConverterType source: aSource destination: aBuffer

	^ native fromAlgorithmic: aConverterType source: aSource destination: aBuffer! !

!ICUConverter methodsFor: 'actions' stamp: 'jvds 2/26/2012 22:18'!
fromType: aConverterType with: aSource

	| buffer length |

	buffer := ByteArray new: aSource size + 10.
	length := native fromAlgorithmic: aConverterType source: aSource destination: buffer.
	^ buffer copyFrom: 1 to: length.! !

!ICUConverter methodsFor: 'actions' stamp: 'JanVanDeSandt 3/2/2012 17:01'!
fromUTF16: aString
	"Convert the Unicode string <aString> into a codepage string."

	| buffer length |

	buffer := ByteArray new: aString size + 10.
	length := native fromUChars: aString destination: buffer.
	^ buffer copyFrom: 1 to: length! !

!ICUConverter methodsFor: 'testing' stamp: 'jvds 2/26/2012 14:08'!
isAmbiguous

	^ native isAmbiguous ! !

!ICUConverter methodsFor: 'testing' stamp: 'JanVanDeSandt 3/4/2012 14:16'!
isFixedWidth
	"Available in ICU version 4.8 or higher."

	^ native isFixedWidth! !

!ICUConverter methodsFor: 'accessing' stamp: 'jvds 2/26/2012 15:28'!
name

	^ native name! !

!ICUConverter methodsFor: 'actions' stamp: 'JanVanDeSandt 3/4/2012 14:45'!
toType: aConverterType source: aSource destination: aBuffer

	^ native toAlgorithmic: aConverterType source: aSource destination: aBuffer! !

!ICUConverter methodsFor: 'actions' stamp: 'JanVanDeSandt 3/2/2012 17:09'!
toUTF16: aString
	"Convert the codepage string <aString> into a Unicode string. "

	| buffer length |

	buffer := ByteArray new: aString size * 2.
	length := native toUChars: aString destination: buffer.
	^TwoByteString fromByteArray: buffer charSize: length! !

!ICUConverter methodsFor: 'accessing' stamp: 'jvds 2/26/2012 15:29'!
type

	^ native type! !

!ICUObject class methodsFor: 'as yet unclassified' stamp: 'JanVanDeSandt 3/2/2012 14:06'!
icuInterface

	^ ICUInterface current! !

!ICUObject methodsFor: 'as yet unclassified' stamp: 'JanVanDeSandt 3/2/2012 14:45'!
close

	native notNil
		ifTrue: [ native close. native := nil ]! !

!ICUObject methodsFor: 'initialize-release' stamp: 'JanVanDeSandt 3/2/2012 14:44'!
setNative: aConverter

	native := aConverter! !
ICUConstants initialize!
ICUConverter initialize!
ICUModules initialize!
